{{
        Spinning Propeller Logo
        Version 001

        Michael Park

}}

CON

  _clkmode = xtal2 + pll8x       ' enable external clock range 5-10MHz and pll times 8
  _xinfreq = 10_000_000 + 0000   ' set frequency to 10 MHZ
  _stack   = 28

  plocksPerPixel = 16
  nPixels       = 128
  nLines        = 160
  
  colorBurstFrq = 3_579_545.0                           ' NTSC color clock frequency in Hz
  plocksPerSecond = colorBurstFrq * 16.0
  linePlocks    = trunc(0.000_063_5 * plocksPerSecond) ' total scanline duration (in plocks)
  hsyncPlocks   = trunc(0.000_010_9 * plocksPerSecond) ' hsync duration
  visiblePlocks = linePlocks - hsyncPlocks              ' duration of visible portion of line

  backPorchPlocks = (visiblePlocks - nPixels * plocksPerPixel) / 2
  frontPorchPlocks = visiblePlocks - nPixels * plocksPerPixel - backPorchPlocks

  totalLines    = 262           ' non-interlaced
  syncLines     = 18
  visibleLines  = totalLines - syncLines
  overscanLines1 = (visibleLines - nLines) / 2
  overscanLines2 = visibleLines - nLines - overscanLines1

PUB Start | i, j, f, d, p, q
  long[@pVclock] := @vclock
  cognew(@HEL_Video_Driver_Entry, @data)

  f := 0
  d := 1
  repeat
    Vsync( 2 )
    p := @data + 40 * 128 + 32
    q := @frame0 + f * 64 * 20
    repeat 20
      longmove( p, q, 16 ) ' 64 bytes
      p += 128
      q += 64
    f += d
    if f < 0
      f := 0
      d := 1
    elseif f > 7
      f := 7
      d := -1

VAR
  long  vclock
  long  prevClock

PRI Vsync( n )
  repeat n
    repeat while prevClock == vclock
    prevClock := vclock

DAT
                        org $000                ' set the code emission for COG add $000

HEL_Video_Driver_Entry

             ' VCFG: setup Video Configuration register and 3-bit TV DAC pins to outputs
             
              movs      vcfg, #%0000_0111       ' pinmask  (pin31 ->0000_0111<-pin24), only want lower 3-bits
              movd      vcfg, #3                ' pingroup (Hydra uses group 3, pins 24-31)
              movi      vcfg, #%0_10_1_01_000   ' controls overall setting, we want baseband video on bottom
                                                '  nibble, 2-bit color, enable chroma on broadcast & baseband
              or        dira, tvport_mask       ' set DAC pins to output on pins 24, 25, 26
               
              ' CTRA: setup Frequency to Drive Video
                       
              movi      ctra,#%00001_111        ' pll internally routed to Video, PHSx+=FRQx (mode 1) + pll(16x)
                                                ' needn't set D,S fields since they set pin A/B I/Os,
                                                '  but mode 1 is internal, thus irrelvant
              mov       frqa, blah              ' FREQ such that the final counter output is NTSC and the PLL output is 16*NTSC 
              mov       r2, #5                  ' how often to animate scrolling pixels
              
Next_Frame
              rdlong    temp, pVclock
              add       temp, #1
              wrlong    temp, pVclock

              ' start of new frame of 262 scanlines, no overscan no half line, to hell with the NTSC/PAL spec!

              ' VSYNC Pulse (Vertical Sync)
              ' 18 scanlines: 6 'high syncs', 6 'low syncs', and finally another 6 'high syncs'
              ' refer to NTSC spec, but this makes up the equalization pulses needed for a vsync
              call      #VsyncHigh
              call      #VsyncLow
              call      #VsyncHigh
              mov       r1, #overscanLines1
              call      #Overscan
              call      #Visible
              mov       r1, #overscanLines2
              call      #Overscan
              jmp       #Next_Frame
              
' Horizontal Scanline Loop
Visible
              mov       r1, #nLines
              mov       ptr, par

Next_Scanline
              mov       vscl, sclHsync          ' set the video scale register such that 16 serial pixels take
                                                '  39 color clocks (the total time of the hsync and burst)
              waitvid   colorsHsync, pixelsHsync ' send out the pixels and the colors to create the hsync pulse

              mov       vscl, sclBackPorch
              waitvid   colorsBorder, #0

              mov       vscl, scl4Pixels

              mov       r3, #nPixels/4
:loop
              rdlong    colors4Pixels, ptr
              waitvid   colors4Pixels, pixels4Pixels
              add       ptr, #4
              djnz      r3, #:loop
              
              mov       vscl, sclFrontPorch
              waitvid   colorsBorder, #0

              djnz      r1, #Next_Scanline      ' are we done with the active scan portion of the frame?

Visible_Ret   ret
                        
' Generate 'HIGH' vsync signal for 6 horizontal lines.
VsyncHigh               
              mov       r1, #6
               
:loop         mov       vscl, sclHsync
              waitvid   colorsHsync, pixelsVsyncHigh1

              mov       vscl, sclVisible
              waitvid   colorsHsync, pixelsVsyncHigh2
              djnz      r1, #:loop
VsyncHigh_Ret ret

' Generate 'LOW' vsync signal for 6 horizontal lines.
VsyncLow
              mov       r1, #6
              
:loop         mov       vscl, sclHsync
              waitvid   colorsHsync, pixelsVsyncLow1

              mov       vscl, sclVisible
              waitvid   colorsHsync, pixelsVsyncLow2
              djnz      r1, #:loop

VsyncLow_Ret ret


Overscan
'       do r1 times:
        
              ' HSYNC 10.9us (Horizontal Sync) including color burst
:loop         mov       vscl, sclHsync          ' set the video scale register such that 16 serial pixels take
                                                '  39 color clocks (the total time of the hsync and burst)
              waitvid   colorsHsync, pixelsHsync ' send out the pixels and the colors to create the hsync pulse
              ' HVIS 52.6us (Visible Scanline) draw 16 huge pixels
              mov       vscl, sclVisible
              waitvid   colorsBorder, #0
              djnz      r1, #:loop
Overscan_Ret  ret                             


'//////////////////////////////////////////////////////////////////////////////
' VARIABLE DECLARATIONS
'//////////////////////////////////////////////////////////////////////////////

' general purpose registers
r0            long              $0                             
r1            long              $0
r2            long              $0
r3            long              $0
ptr           long              $0
ptr0          long              $7000

' tv output DAC port bit mask
tvport_mask   long              %0000_0111 << 24' Hydra DAC is on bits 24, 25, 26

sclHsync      long              ((hsyncPlocks >> 4) << 12) + hsyncPlocks
sclVisible    long              visiblePlocks
sclBackPorch  long              backPorchPlocks
sclFrontPorch long              frontPorchPlocks
scl4Pixels    long              (plocksPerPixel << 12) + 4 * plocksPerPixel

' hsync colors (4, 8-bit values, each represent a color in the format chroma shift, chroma modulatation enable, luma | C3 C2 C1 C0 | M | L2 L1 L0 |
                                                '3  2  1  0   <- color indexes
colorsHsync   long              $00_00_02_8A    ' SYNC (3) / SYNC (2) / BLACKER THAN BLACK (1) / COLOR BURST (0)

' hsync pixels
                                                ' BP  |BURST|BW|    SYNC      |FP| <- Key BP = Back Porch, Burst = Color Burst, BW = Breezway, FP = Front Porch
pixelsHsync   long              %%1_1_0_0_0_0_1_2_2_2_2_2_2_2_1_1

' the colors used, 4 of them always
                                               ' red, color 3 | dark blue, color 2 | blue, color 1 | light blue, color 0
colors4Pixels long              $4c_0A_0B_0C   ' each 2-bit pixel below references one of these 4 colors, (msb) 3,2,1,0 (lsb)

colorsBorder  long              $00_00_00_07   ' each 2-bit pixel below references one of these 4 colors, (msb) 3,2,1,0 (lsb)

' the pixel pattern
pixels4Pixels long              %%3333_2222_1111_3210   ' 16-pixels, read low to high is rendered left to right, 2 bits per pixel
                                                        ' the numbers 0,1,2,3 indicate the "colors" to use for the pixels, the colors
                                                        ' are defined by a single byte each with represents the chroma shift, modulation,
                                                        ' and luma

                                

' vsync pulses 6x High, 6x Low, 6x High
' the vertical sync pulse according to the NTSC spec should be composed of a series
' of pulses called the pre-equalization, serration pulses (the VSYNC pulse itself), and the post-equalization pulses
' there are 6 pulses of each, and they more or less inverted HSYNC, followed by 6 HSYNC pulses, followed by 6 more inverted HSYNC pulses.
' this keeps the horizontal timing circutry locked as well as allows the 60 Hz VSYNC filter to catch the "vsync" event.
' the values 1,2 index into "colors" that represent sync and blacker than black.
' so the definitions below help with generated the "high" and "low" dominate HSYNC timed pulses which are combined
' to generated the actual VSYNC pulse, refer to NTSC documentation for more details.
pixelsVsyncHigh1        long    %%1_1_1_1_1_1_1_1_1_1_1_2_2_2_1_1  
pixelsVsyncHigh2        long    %%1_1_1_1_1_1_1_1_1_1_1_1_1_1_1_1

pixelsVsyncLow1         long    %%2_2_2_2_2_2_2_2_2_2_2_2_2_2_1_1
pixelsVsyncLow2         long    %%1_2_2_2_2_2_2_2_2_2_2_2_2_2_2_2

temp          long              0
blah          long              trunc(colorBurstFrq / 80_000_000.0 * 4_294_967_896.0)

DAT
pVclock long  0
data    FILE  "logo128x160d.dat"
frame0  FILE  "prop64x20d_00.dat"
frame1  FILE  "prop64x20d_01.dat"
frame2  FILE  "prop64x20d_02.dat"
frame3  FILE  "prop64x20d_03.dat"
frame4  FILE  "prop64x20d_04.dat"
frame5  FILE  "prop64x20d_05.dat"
frame6  FILE  "prop64x20d_06.dat"
frame7  FILE  "prop64x20d_07.dat"
        